/* Test all the functions provided by FastThreads */

#include <stdio.h>
#include "thread.h"
#include "synch.h"

#define FALSE 0
#define TRUE 1

void dummy1(), dummy2(), dummy3();

void CloseTest ()
  {
  int i = 1000; 

  for (i = 0; i < 1000; i++)
	{
	dummy1();
	dummy2();
	dummy3();
	}
  }

/* I don't care how many processors I have! */

void SpinLockTest (sl)
  SpinLock *sl;
  {
  int i;

  for (i = 1000; --i;)
	{
	SpinLockAcquire(sl);
	SpinLockRelease(sl);
	}
  }

void LockTest (l)
  Lock *l;
  {
  int i, j;

  for (i = 100; --i;)
	{
	LockAcquire(l);
	LockRelease(l);
	for (j = 10; --j;)
	  ;
	}
  }

void SpinBarrierTest (sb)
  SpinBarrier *sb;
  {
  int i, j;

  for (i = 100; --i;)
	{
	j = SpinBarrierCheckIn(sb);
	SpinBarrierWait(sb, j);
	}
  }

void BarrierTest (b)
  Barrier *b;
  {
  int i, j;

  for (i = 100; --i;)
	{
	j = BarrierCheckIn(b);
	BarrierWait(b, j);
	}
  }

void SpinPing (sl, sc1, sc2, reached)
  SpinLock *sl;
  SpinCondition *sc1, *sc2;
  int *reached;
  {
  int i;

  for (i = 100; --i;)
	{
	SpinLockAcquire(sl);
	*reached = TRUE;
	SpinConditionSignal(sc2);
	SpinConditionWaitLock(sc1, sl);
	SpinLockRelease(sl);
	}
  }

void SpinPong (sl, sc1, sc2, reached)
  SpinLock *sl;
  SpinCondition *sc1, *sc2;
  int *reached;
  {
  int i;

  for (i = 100; --i;)
	{
	SpinLockAcquire(sl);
	if (!*reached)
		SpinConditionWaitLock(sc2, sl);
	*reached = FALSE;
	SpinConditionSignal(sc1);
	SpinLockRelease(sl);
	}
  }

void Ping (l, c1, c2, reached)
  Lock *l;
  Condition *c1, *c2;
  int *reached;
  {
  int i;

  for (i = 100; --i;)
	{
	LockAcquire(l);
	*reached = TRUE;
	ConditionSignal(c2);
	ConditionWaitLock(c1, l);
	LockRelease(l);
	}
  }

void Pong (l, c1, c2, reached)
  Lock *l;
  Condition *c1, *c2;
  int *reached;
  {
  int i;

  for (i = 100; --i;)
	{
	LockAcquire(l);
	if (!*reached)
		ConditionWaitLock(c2, l);
	*reached = FALSE;
	ConditionSignal(c1);
	LockRelease(l);
	}
  }

void SpinTest ()
  {
  Thread *t;
  SpinLock spinLock;
  SpinBarrier spinBarrier;

  printf("Starting SpinLockTest\n");
  fflush(stdout);

  SpinLockInit(&spinLock, "SpinLockTest");
  t = ThreadStart(SpinLockTest, JOIN, NULL, 1, &spinLock);
  SpinLockTest(&spinLock);
  ThreadJoin(t);

  printf("Starting SpinBarrierTest\n");
  fflush(stdout);

  SpinBarrierInit(&spinBarrier, 2, "SpinBarrierTest");
  t = ThreadStart(SpinBarrierTest, JOIN, NULL, 1, &spinBarrier);
  SpinBarrierTest(&spinBarrier);
  ThreadJoin(t);

  printf("Done with Spin Tests\n");
  fflush(stdout);
  }

void BlockTest ()
  {
  Thread *t;
  Lock lock;
  Barrier barrier;
  SpinLock spinLock;
  SpinCondition spinCondition1, spinCondition2;
  Condition condition1, condition2;
  int reached;

  printf("Starting LockTest\n");
  fflush(stdout);

  LockInit(&lock, "LockTest");
  t = ThreadStart(LockTest, JOIN, NULL, 1, &lock);
  LockTest(&lock);
  ThreadJoin(t);

  printf("Starting BarrierTest\n");
  fflush(stdout);

  BarrierInit(&barrier, 2, "BarrierTest");
  t = ThreadStart(BarrierTest, JOIN, NULL, 1, &barrier);
  BarrierTest(&barrier);
  ThreadJoin(t);

  printf("Starting SpinPongTest\n");
  fflush(stdout);

  SpinLockInit(&spinLock, "PongTest");
  SpinConditionInit(&spinCondition1, "SpinCondition1");
  SpinConditionInit(&spinCondition2, "SpinCondition2");
  reached = FALSE;
  t = ThreadStart(SpinPing, JOIN, NULL, 4, 
			&spinLock, &spinCondition1, &spinCondition2, &reached);
  SpinPong(&spinLock, &spinCondition1, &spinCondition2, &reached);
  ThreadJoin(t);

  printf("Starting PongTest\n");
  fflush(stdout);

  ConditionInit(&condition1, "condition1");
  ConditionInit(&conditiowi, "condition2");
  reached = FALSE;
  t = ThreadStart(Ping, JOIN, NULL, 4, 
			&lock, &condition1, &condition2, &reached);
  Pong(&lock, &condition1, &condition2, &reached);
  ThreadJoin(t);

  printf("Done with Block Tests\n");
  fflush(stdout);
  }

Init()
  {
  (void)ThreadStart(Main, NOJOIN, (char *)NULL, 0);
  }

Main ()
  {
  if (numProcessors > 1)
    SpinTest();
  BlockTest();
  CloseTest();
  ProgramDone();
  }
